#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This software is licensed as described in the README.rst and LICENSE files,
# which you should have received as part of this distribution.

import RPi.GPIO as GPIO
import time
import logging


class Doorman(object):
    args = []
    sensor_pin = 16
    sensor_state = 0
    door_state = 2 # Set unknown value, due to initial reading
    door_closed = 0
    door_open = 1

    def __init__(self, args=[]):
        if args:
            self.args = args

        # Prepare logging configuration
        logconfig = {
            'filename': '/tmp/doorman.log',
            'level': logging.INFO,
            'format': '%(asctime)s %(levelname)-8s %(name)s: %(message)s',
        }

        if hasattr(args, 'log') and args.log:
            logconfig['filename'] = args.log

        if hasattr(args, 'debug') and args.debug:
            logconfig['level'] = logging.DEBUG

        logging.basicConfig(**logconfig)
        logging.info('Doorman is at your service')

        if hasattr(args, 'pin') and args.pin:
            self.sensor_pin = args.pin
        logging.debug('Sensor PIN: %s', self.sensor_pin)

        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(self.sensor_pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)
        self.sensor_read()

    def get_door_state(self):
        if self.door_state == self.door_open:
            return 'Door is open'
        else:
            return 'Door is closed'

    def notify(self):
        if hasattr(self.args, 'notify') and self.args.notify:
            try:
                import requests
            except ImportError:
                logger.error('Can not import requests library. Make sure it is installed!')
            else:
                r = requests.post("http://127.0.0.1:8922/doorman_update",
                        data = {
                            "user": self.args.notify,
                            "msg": self.get_door_state()
                            }
                        )
                log = 'Notification response code: %s (user: %s, msg: %s)' % (r.status_code, self.args.notify,
                        self.get_door_state())

                if r.status_code != requests.codes.ok:
                    logging.error(log)
                else:
                    logging.debug(log)

    def callback_sensor_read(self):
        logging.info(self.get_door_state())

    def sensor_read(self):
        self.sensor_state = GPIO.input(self.sensor_pin)

        if self.sensor_state != self.door_state:
            logging.debug('Changing sensor state from %s to %s' % (self.door_state, self.sensor_state))
            self.door_state = self.sensor_state
            self.callback_sensor_read()
            self.notify()

    def sense(self):
        try:
            logging.debug('Doorman is starting to watch the door status')
            while True:
                self.sensor_read()
                time.sleep(0.1)

        except KeyboardInterrupt: # If CTRL+C is pressed, exit cleanly:
            logging.info('Thank you for using my service, your Doorman!')
            GPIO.cleanup()




import unittest
from rpi_doorman.rpi_doorman import Doorman

class DoormanObjectTestCase(unittest.TestCase):
    def setUp(self):
        self.d = Doorman()

    def tearDown(self):
        self.d = None

    def test_constants(self):
        self.assertEqual(self.d.door_open, 1)
        self.assertEqual(self.d.door_closed, 0)

    def test_get_door_state(self):
        self.d.door_state = 1
        self.assertEqual(self.d.get_door_state(), 'Door is open')
        self.d.door_state = 0
        self.assertEqual(self.d.get_door_state(), 'Door is closed')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This software is licensed as described in the README.rst and LICENSE files,
# which you should have received as part of this distribution.

import argparse
from rpi_doorman import Doorman


def setup_args():
    ap = argparse.ArgumentParser(prog='doorman',
            description='RPi.Doorman is using adafruit magnetic contact switch (door sensor), will permanently sense for door status and its changes.'
            )
    ap.add_argument('-c', action='store_true', help='Current door status will be shown.')
    ap.add_argument('--notify', type=str, help='Jabber ID to notify via Ludolph webhook with sensor state change (POST).')
    ap.add_argument('-p', '--pin', type=int, help='Pin number, for GPIO magnetic contact switch (door sensor).')
    ap.add_argument('-l', '--log', type=str, help='Log path, where logs will be stored.')
    ap.add_argument('-d', '--debug', action='store_true', help='Debug mode, wheather store debug info in log.')

    return ap.parse_args()


def main():
    args = setup_args()
    d = Doorman(args)

    if hasattr(args, 'c') and args.c:
        print(d.get_door_state())
    else:
        d.sense()


if __name__ == "__main__":
    # execute only if run as a script
    main()



import unittest
from test_doorman_object import DoormanObjectTestCase

def create_suite():
    test_suite = unittest.TestSuite()
    test_suite.addTest(DoormanObjectTestCase())
    return test_suite


if __name__ == '__main__':
   suite = create_suite()

   runner=unittest.TextTestRunner()
   runner.run(suite)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This software is licensed as described in the README.rst and LICENSE
# files, which you should have received as part of this distribution.

import os
import sys
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# noinspection PyPep8Naming
from rpi_doorman import __version__ as VERSION

read = lambda fname: open(os.path.join(os.path.dirname(__file__), fname)).read()

DEPS = ['RPi.GPIO',]

CLASSIFIERS = [
    'Environment :: Console',
    'Intended Audience :: System Administrators',
    'Intended Audience :: Developers',
    'Operating System :: Unix',
    'Operating System :: POSIX :: Linux',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Development Status :: 3 - Alpha',
    'Topic :: Utilities'
]

packages = [
    'rpi_doorman',
]

setup(
    name='rpi-doorman',
    version=VERSION,
    description='Door state monitor',
    long_description=read('README.rst'),
    author='Richard Kellner',
    author_email='richard.kellner [at] gmail.com',
    url='https://github.com/ricco386/rpi-doorman/',
    license='MIT',
    packages=packages,
    scripts=['bin/rpi-doorman'],
    install_requires=DEPS,
    platforms='any',
    classifiers=CLASSIFIERS,
    include_package_data=True
)
